#include "fddfam.h"
#include <fam.h>
#include <assert.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

using namespace Foldroid;
using namespace std;

Fam::Fam()
{
  instance = this;

  int fam_error = FAMOpen (&m_fam_connection);
  if (fam_error)
    {
      fprintf (stderr, "Fam: can't setup fam connection\n");
      exit (1);
    }
}

Fam::~Fam()
{
  instance = 0;
}

void
Fam::add_file (FamObserver *observer, const std::string& filename)
{
  string name = filename;
  if (name[0] != '/')
    {
      char buffer[1024];
      string dir = getcwd (buffer, 1024);
      name = dir + "/" + name;
    }

  if (m_observers[name].empty()) /* need to setup a new fam request, then */
    {
      int fam_error;
      fam_error = FAMMonitorFile (&m_fam_connection, name.c_str(), &m_requests[name], observer);
      if (fam_error)
        {
          fprintf (stderr, "Fam: can't monitor file\n");
          exit (1);
        }
    }
  m_observers[name].push_back (observer);
}

void
Fam::remove_file (FamObserver *observer)
{
  map<string,list<FamObserver *> >::iterator oi = m_observers.begin();
  while (oi != m_observers.end())
    {
      list<FamObserver *>& observers = oi->second;
      bool empty_before = observers.empty();
      observers.remove (observer);
      bool empty_after = observers.empty();

      if (empty_before != empty_after)
        {
          int fam_error = FAMCancelMonitor (&m_fam_connection, &m_requests[oi->first]);
          if (fam_error)
            {
              fprintf (stderr, "Fam: can't unmonitor file\n");
              exit (1);
            }
        }

      oi++;
    }
}

void
Fam::check ()
{
  while (FAMPending (&m_fam_connection))
    {
      FAMEvent fam_event;
      FAMNextEvent (&m_fam_connection, &fam_event);
      list<FamObserver *>& olist = m_observers[fam_event.filename];
      for (list<FamObserver *>::iterator oli = olist.begin(); oli != olist.end(); oli++)
        (*oli)->notify_file_changed (fam_event.filename);
    }
}

Fam *Fam::instance = 0;

Fam *Fam::the()
{
  assert (instance);
  return instance;
}
